#compdef grid
# # DO NOT EDIT. Autogenerated with:
# _GRID_COMPLETE=source_zsh grid > grid/autocomplete/complete.zsh
# See: https://click.palletsprojects.com/en/7.x/bashcomplete/

_grid_completion() {
    local -a completions
    local -a completions_with_descriptions
    local -a response
    (( ! $+commands[grid] )) && return 1

    response=("${(@f)$( env COMP_WORDS="${words[*]}" \
                        COMP_CWORD=$((CURRENT-1)) \
                        _GRID_COMPLETE="complete_zsh" \
                        grid )}")

    for key descr in ${(kv)response}; do
      if [[ "$descr" == "_" ]]; then
          completions+=("$key")
      else
          completions_with_descriptions+=("$key":"$descr")
      fi
    done

    if [ -n "$completions_with_descriptions" ]; then
        _describe -V unsorted completions_with_descriptions -U
    fi

    if [ -n "$completions" ]; then
        compadd -U -V unsorted -a completions
    fi
    compstate[insert]="automenu"
}

compdef _grid_completion grid;
