# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""


import pprint
import re  # noqa: F401
from typing import TYPE_CHECKING

import six

from grid.openapi.configuration import Configuration

if TYPE_CHECKING:
    from datetime import datetime
    from grid.openapi.models import *

class V1ClusterSpec(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cluster_type': 'V1ClusterType',
        'cost_factor': 'str',
        'desired_state': 'V1ClusterState',
        'driver': 'V1ClusterDriver',
        'gridlet_version': 'str',
        'padding_factor': 'str',
        'pause_automation': 'bool',
        'performance_profile': 'V1ClusterPerformanceProfile',
        'user_id': 'str'
    }

    attribute_map = {
        'cluster_type': 'clusterType',
        'cost_factor': 'costFactor',
        'desired_state': 'desiredState',
        'driver': 'driver',
        'gridlet_version': 'gridletVersion',
        'padding_factor': 'paddingFactor',
        'pause_automation': 'pauseAutomation',
        'performance_profile': 'performanceProfile',
        'user_id': 'userId'
    }

    def __init__(self, cluster_type: 'V1ClusterType' = None, cost_factor: 'str' = None, desired_state: 'V1ClusterState' = None, driver: 'V1ClusterDriver' = None, gridlet_version: 'str' = None, padding_factor: 'str' = None, pause_automation: 'bool' = None, performance_profile: 'V1ClusterPerformanceProfile' = None, user_id: 'str' = None, _configuration=None):  # noqa: E501
        """V1ClusterSpec - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._cluster_type = None
        self._cost_factor = None
        self._desired_state = None
        self._driver = None
        self._gridlet_version = None
        self._padding_factor = None
        self._pause_automation = None
        self._performance_profile = None
        self._user_id = None
        self.discriminator = None

        if cluster_type is not None:
            self.cluster_type = cluster_type
        if cost_factor is not None:
            self.cost_factor = cost_factor
        if desired_state is not None:
            self.desired_state = desired_state
        if driver is not None:
            self.driver = driver
        if gridlet_version is not None:
            self.gridlet_version = gridlet_version
        if padding_factor is not None:
            self.padding_factor = padding_factor
        if pause_automation is not None:
            self.pause_automation = pause_automation
        if performance_profile is not None:
            self.performance_profile = performance_profile
        if user_id is not None:
            self.user_id = user_id

    @property
    def cluster_type(self) -> 'V1ClusterType':
        """Gets the cluster_type of this V1ClusterSpec.  # noqa: E501


        :return: The cluster_type of this V1ClusterSpec.  # noqa: E501
        :rtype: V1ClusterType
        """
        return self._cluster_type

    @cluster_type.setter
    def cluster_type(self, cluster_type: 'V1ClusterType'):
        """Sets the cluster_type of this V1ClusterSpec.


        :param cluster_type: The cluster_type of this V1ClusterSpec.  # noqa: E501
        :type: V1ClusterType
        """

        self._cluster_type = cluster_type

    @property
    def cost_factor(self) -> 'str':
        """Gets the cost_factor of this V1ClusterSpec.  # noqa: E501

        costController needs for calculating costOfGoods - string since it's decimal. Note: This value is stored in a separate column in the database, not with the rest of the cluster JSON in spec.  # noqa: E501

        :return: The cost_factor of this V1ClusterSpec.  # noqa: E501
        :rtype: str
        """
        return self._cost_factor

    @cost_factor.setter
    def cost_factor(self, cost_factor: 'str'):
        """Sets the cost_factor of this V1ClusterSpec.

        costController needs for calculating costOfGoods - string since it's decimal. Note: This value is stored in a separate column in the database, not with the rest of the cluster JSON in spec.  # noqa: E501

        :param cost_factor: The cost_factor of this V1ClusterSpec.  # noqa: E501
        :type: str
        """

        self._cost_factor = cost_factor

    @property
    def desired_state(self) -> 'V1ClusterState':
        """Gets the desired_state of this V1ClusterSpec.  # noqa: E501


        :return: The desired_state of this V1ClusterSpec.  # noqa: E501
        :rtype: V1ClusterState
        """
        return self._desired_state

    @desired_state.setter
    def desired_state(self, desired_state: 'V1ClusterState'):
        """Sets the desired_state of this V1ClusterSpec.


        :param desired_state: The desired_state of this V1ClusterSpec.  # noqa: E501
        :type: V1ClusterState
        """

        self._desired_state = desired_state

    @property
    def driver(self) -> 'V1ClusterDriver':
        """Gets the driver of this V1ClusterSpec.  # noqa: E501


        :return: The driver of this V1ClusterSpec.  # noqa: E501
        :rtype: V1ClusterDriver
        """
        return self._driver

    @driver.setter
    def driver(self, driver: 'V1ClusterDriver'):
        """Sets the driver of this V1ClusterSpec.


        :param driver: The driver of this V1ClusterSpec.  # noqa: E501
        :type: V1ClusterDriver
        """

        self._driver = driver

    @property
    def gridlet_version(self) -> 'str':
        """Gets the gridlet_version of this V1ClusterSpec.  # noqa: E501


        :return: The gridlet_version of this V1ClusterSpec.  # noqa: E501
        :rtype: str
        """
        return self._gridlet_version

    @gridlet_version.setter
    def gridlet_version(self, gridlet_version: 'str'):
        """Sets the gridlet_version of this V1ClusterSpec.


        :param gridlet_version: The gridlet_version of this V1ClusterSpec.  # noqa: E501
        :type: str
        """

        self._gridlet_version = gridlet_version

    @property
    def padding_factor(self) -> 'str':
        """Gets the padding_factor of this V1ClusterSpec.  # noqa: E501

        costController needs for calculating paddingValue - string since it's decimal Note: This value is stored in a separate column in the database, not with the rest of the cluster JSON in spec.  # noqa: E501

        :return: The padding_factor of this V1ClusterSpec.  # noqa: E501
        :rtype: str
        """
        return self._padding_factor

    @padding_factor.setter
    def padding_factor(self, padding_factor: 'str'):
        """Sets the padding_factor of this V1ClusterSpec.

        costController needs for calculating paddingValue - string since it's decimal Note: This value is stored in a separate column in the database, not with the rest of the cluster JSON in spec.  # noqa: E501

        :param padding_factor: The padding_factor of this V1ClusterSpec.  # noqa: E501
        :type: str
        """

        self._padding_factor = padding_factor

    @property
    def pause_automation(self) -> 'bool':
        """Gets the pause_automation of this V1ClusterSpec.  # noqa: E501


        :return: The pause_automation of this V1ClusterSpec.  # noqa: E501
        :rtype: bool
        """
        return self._pause_automation

    @pause_automation.setter
    def pause_automation(self, pause_automation: 'bool'):
        """Sets the pause_automation of this V1ClusterSpec.


        :param pause_automation: The pause_automation of this V1ClusterSpec.  # noqa: E501
        :type: bool
        """

        self._pause_automation = pause_automation

    @property
    def performance_profile(self) -> 'V1ClusterPerformanceProfile':
        """Gets the performance_profile of this V1ClusterSpec.  # noqa: E501


        :return: The performance_profile of this V1ClusterSpec.  # noqa: E501
        :rtype: V1ClusterPerformanceProfile
        """
        return self._performance_profile

    @performance_profile.setter
    def performance_profile(self, performance_profile: 'V1ClusterPerformanceProfile'):
        """Sets the performance_profile of this V1ClusterSpec.


        :param performance_profile: The performance_profile of this V1ClusterSpec.  # noqa: E501
        :type: V1ClusterPerformanceProfile
        """

        self._performance_profile = performance_profile

    @property
    def user_id(self) -> 'str':
        """Gets the user_id of this V1ClusterSpec.  # noqa: E501


        :return: The user_id of this V1ClusterSpec.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id: 'str'):
        """Sets the user_id of this V1ClusterSpec.


        :param user_id: The user_id of this V1ClusterSpec.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1ClusterSpec, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'V1ClusterSpec') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ClusterSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other: 'V1ClusterSpec') -> bool:
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ClusterSpec):
            return True

        return self.to_dict() != other.to_dict()
