# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""


import pprint
import re  # noqa: F401
from typing import TYPE_CHECKING

import six

from grid.openapi.configuration import Configuration

if TYPE_CHECKING:
    from datetime import datetime
    from grid.openapi.models import *

class Externalv1ExperimentStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cost': 'float',
        'deleted_timestamp': 'datetime',
        'finish_timestamp': 'datetime',
        'hourly_cost': 'float',
        'message': 'str',
        'phase': 'V1ExperimentState',
        'start_timestamp': 'datetime'
    }

    attribute_map = {
        'cost': 'cost',
        'deleted_timestamp': 'deletedTimestamp',
        'finish_timestamp': 'finishTimestamp',
        'hourly_cost': 'hourlyCost',
        'message': 'message',
        'phase': 'phase',
        'start_timestamp': 'startTimestamp'
    }

    def __init__(self, cost: 'float' = None, deleted_timestamp: 'datetime' = None, finish_timestamp: 'datetime' = None, hourly_cost: 'float' = None, message: 'str' = None, phase: 'V1ExperimentState' = None, start_timestamp: 'datetime' = None, _configuration=None):  # noqa: E501
        """Externalv1ExperimentStatus - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._cost = None
        self._deleted_timestamp = None
        self._finish_timestamp = None
        self._hourly_cost = None
        self._message = None
        self._phase = None
        self._start_timestamp = None
        self.discriminator = None

        if cost is not None:
            self.cost = cost
        if deleted_timestamp is not None:
            self.deleted_timestamp = deleted_timestamp
        if finish_timestamp is not None:
            self.finish_timestamp = finish_timestamp
        if hourly_cost is not None:
            self.hourly_cost = hourly_cost
        if message is not None:
            self.message = message
        if phase is not None:
            self.phase = phase
        if start_timestamp is not None:
            self.start_timestamp = start_timestamp

    @property
    def cost(self) -> 'float':
        """Gets the cost of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The cost of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: float
        """
        return self._cost

    @cost.setter
    def cost(self, cost: 'float'):
        """Sets the cost of this Externalv1ExperimentStatus.


        :param cost: The cost of this Externalv1ExperimentStatus.  # noqa: E501
        :type: float
        """

        self._cost = cost

    @property
    def deleted_timestamp(self) -> 'datetime':
        """Gets the deleted_timestamp of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The deleted_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._deleted_timestamp

    @deleted_timestamp.setter
    def deleted_timestamp(self, deleted_timestamp: 'datetime'):
        """Sets the deleted_timestamp of this Externalv1ExperimentStatus.


        :param deleted_timestamp: The deleted_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :type: datetime
        """

        self._deleted_timestamp = deleted_timestamp

    @property
    def finish_timestamp(self) -> 'datetime':
        """Gets the finish_timestamp of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The finish_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._finish_timestamp

    @finish_timestamp.setter
    def finish_timestamp(self, finish_timestamp: 'datetime'):
        """Sets the finish_timestamp of this Externalv1ExperimentStatus.


        :param finish_timestamp: The finish_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :type: datetime
        """

        self._finish_timestamp = finish_timestamp

    @property
    def hourly_cost(self) -> 'float':
        """Gets the hourly_cost of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The hourly_cost of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: float
        """
        return self._hourly_cost

    @hourly_cost.setter
    def hourly_cost(self, hourly_cost: 'float'):
        """Sets the hourly_cost of this Externalv1ExperimentStatus.


        :param hourly_cost: The hourly_cost of this Externalv1ExperimentStatus.  # noqa: E501
        :type: float
        """

        self._hourly_cost = hourly_cost

    @property
    def message(self) -> 'str':
        """Gets the message of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The message of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message: 'str'):
        """Sets the message of this Externalv1ExperimentStatus.


        :param message: The message of this Externalv1ExperimentStatus.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def phase(self) -> 'V1ExperimentState':
        """Gets the phase of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The phase of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: V1ExperimentState
        """
        return self._phase

    @phase.setter
    def phase(self, phase: 'V1ExperimentState'):
        """Sets the phase of this Externalv1ExperimentStatus.


        :param phase: The phase of this Externalv1ExperimentStatus.  # noqa: E501
        :type: V1ExperimentState
        """

        self._phase = phase

    @property
    def start_timestamp(self) -> 'datetime':
        """Gets the start_timestamp of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The start_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._start_timestamp

    @start_timestamp.setter
    def start_timestamp(self, start_timestamp: 'datetime'):
        """Sets the start_timestamp of this Externalv1ExperimentStatus.


        :param start_timestamp: The start_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :type: datetime
        """

        self._start_timestamp = start_timestamp

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Externalv1ExperimentStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'Externalv1ExperimentStatus') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, Externalv1ExperimentStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other: 'Externalv1ExperimentStatus') -> bool:
        """Returns true if both objects are not equal"""
        if not isinstance(other, Externalv1ExperimentStatus):
            return True

        return self.to_dict() != other.to_dict()
